#!/usr/bin/env perl
use strict;
use warnings;

use MIDI::Util qw(midi_dump reverse_dump);
use Data::Dumper::Compact qw(ddc);

my $name = shift || 'length';

print ddc(midi_dump($name));

print ddc(reverse_dump($name, 4));

__END__
volume
length
note
note2number
number2note
patch2number
number2patch
notenum2percussion
percussion2notenum
all_events
midi_events
meta_events
text_events
nontext_meta_events
